<?php

namespace AltSolution\Admin\Backup;

use Illuminate\Database\DatabaseManager as DBM;
use Illuminate\Database\Connection as DBC;
use PDO;

// https://github.com/twitter/mysql/blob/master/client/mysqldump.c#L5445
class Database
{
    /**
     * @var DBC
     */
    private $conn;
    /**
     * @var PDO
     */
    private $pdo;

    public $eol = "\r\n";

    public function __construct(DBM $databaseManager)
    {
        $this->conn = $databaseManager->connection(config('database.default'));
        $this->pdo = $this->conn->getPdo();
    }

    public function createDump()
    {
        $filePath = storage_path('backup/db-'.date('YmdHis').'.sql');
        $fileDir = dirname($filePath);
        if (!is_dir($fileDir)) {
            mkdir($fileDir, 0777, true);
        }
        $file = DumpFile::plain($filePath);
        $file->open();
        $this->dump($file);
        $file->close();

        return $filePath;
    }

    private function dump(DumpFile $file)
    {
        $eol = $this->eol;

        //$this->pdo->exec("USE `ntema`");
        //$this->pdo->exec("USE `pa`");
        // $this->pdo->setAttribute(PDO::ATTR_STRINGIFY_FETCHES, true);

        $this->pdo->exec("SET @OLD_TIME_ZONE=@@TIME_ZONE");
        $this->pdo->exec("SET @@TIME_ZONE='+00:00'");

        //$file->write("-- Host: " . $this->db->host . $eol);
        //$file->write("-- DB name: " . $this->db->name . $eol);

        $file->write("/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;$eol");
        $file->write("/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;$eol");
        $file->write("/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;$eol");
        $file->write("/*!40101 SET NAMES utf8 */;$eol");
        $file->write("/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;$eol");
        $file->write("/*!40103 SET TIME_ZONE='+00:00' */;$eol");
        $file->write("/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;$eol");
        $file->write("/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;$eol");
        $file->write("/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;$eol");
        $file->write("/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;$eol$eol");

        $tables = $this->listTables();
        foreach ($tables as $table) {
            $this->dumpTable($file, $table);
        }

        $file->write("/*!40103 SET TIME_ZONE=@OLD_TIME_ZONE */;$eol");
        $file->write("/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;$eol");
        $file->write("/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;$eol");
        $file->write("/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;$eol");
        $file->write("/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;$eol");
        $file->write("/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;$eol");
        $file->write("/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;$eol");
        $file->write("/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;$eol$eol");
        $file->write('-- Dump completed on '.date('Y-m-d H:i:s').$eol.$eol);

        $this->pdo->exec('SET @@TIME_ZONE=@OLD_TIME_ZONE');
    }

//    private function listTables()
//    {
//        $stmt = $this->pdo->query('SELECT DATABASE()');
//        $dbname = $stmt->fetch(PDO::FETCH_COLUMN);
//        if ($dbname === false) {
//            throw new \Exception('Failed get current database name');
//        }
//
//        // TODO: support for 'VIEW'
//        $stmt = $this->pdo->prepare('SHOW FULL TABLES WHERE Table_Type = ?');
//        $ok = $stmt->execute(['BASE TABLE']);
//        if (!$ok) {
//            throw new \Exception('Failed get list of tables in database');
//        }
//
//        $res = $stmt->fetchAll(PDO::FETCH_ASSOC);
//
//        $result = [];
//        foreach ($res as $r) {
//            $tableName = $r['Tables_in_' . $dbname];
//            $result[] = $tableName;
//        }
//
//        return $result;
//    }

    private function listTables()
    {
        $stmt = $this->pdo->query('SHOW TABLE STATUS');
        $res = $stmt->fetchAll(PDO::FETCH_ASSOC);

        /*
         [ "Name", "Engine", "Version", "Row_format", "Rows", "Avg_row_length",
         "Data_length", "Max_data_length", "Index_length", "Data_free", "Auto_increment",
         "Create_time", "Update_time", "Check_time", "Collation", "Checksum", "Create_options", "Comment" ]
         */

        return $res;
    }

    private function dumpTable(DumpFile $file, $tableInfo)
    {
        $eol = $this->eol;
        $table = $tableInfo['Name'];

        $stmt = $this->pdo->query("SHOW CREATE TABLE `$table`");
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        $createQuery = $row['Create Table'];

        $file->write("--$eol");
        $file->write("-- Table structure for table `$table`$eol");
        $file->write("--$eol$eol");
        $file->write("DROP TABLE IF EXISTS `$table`;$eol");
        $file->write("/*!40101 SET @saved_cs_client     = @@character_set_client */;$eol");
        $file->write("/*!40101 SET character_set_client = utf8 */;$eol");
        $file->write("$createQuery;$eol");
        $file->write("/*!40101 SET character_set_client = @saved_cs_client */;$eol$eol");

        $file->write("--$eol");
        $file->write("-- Dumping data for table `$table`$eol");
        $file->write("--$eol$eol");


        $stmt = $this->pdo->query("SELECT COUNT(*) FROM `$table`");
        $total = $stmt->fetch(PDO::FETCH_COLUMN);
        if ($total === false) {
            throw new \Exception('Failed get count');
        }

        $file->write("LOCK TABLES `$table` WRITE;$eol");
        if ($tableInfo['Engine'] == 'InnoDB') {
            $file->write("/*!40000 ALTER TABLE `$table` DISABLE KEYS */;$eol");
        }

        if ($total > 0) {
            // TODO: chunk
            $stmt = $this->pdo->query("SELECT * FROM `$table`");
            // TODO: no values
            $file->write("INSERT INTO `$table` VALUES ");

            $firstValue = true;
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
                $cols = $this->escapeRow($row);
                if ($firstValue) {
                    $firstValue = false;
                } else {
                    $file->write(',');
                }
                $file->write('(' . implode(',', $cols) . ')');
            }
            $file->write(";$eol");
        }

        if ($tableInfo['Engine'] == 'InnoDB') {
            $file->write("/*!40000 ALTER TABLE `$table` ENABLE KEYS */;$eol");
        }
        $file->write("UNLOCK TABLES;$eol$eol");
    }

    private function escapeRow($row)
    {
        $cols = [];
        foreach ($row as $col) {
            // preg_match('/^(?:[-+])?(?:\d+|\.\d+|\d+\.\d+)(?:[eE][-+]?\d+)?$/', $col)
            if (is_int($col) || is_float($col)) {
                $value = $col;
            } elseif ($col === null) {
                $value = 'NULL';
            } else {
                $value = $this->pdo->quote($col);
            }
            $cols[] = $value;
        }
        return $cols;
    }
}
