<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Cron\RepositoryInterface;
use AltSolution\Admin\Models\Cron;
use AltSolution\Admin\Forms\CronForm;
use Illuminate\Http\Request;

class CronController extends Controller
{
    public function getIndex(Request $request)
    {
        $this->authorize('permission', 'cron.list');

        $filter = [
            'q' => $request->input('q'),
            'sort' => $request->input('sort'),
        ];

        $items = Cron::query()->get();

        $this->layout
            ->setMenuSection('cron')
            ->setTitle(trans('admin::cron.list'));
        return view('admin::cron.list', compact('items'));
    }

    public function getEdit($id = null)
    {
        $this->authorize('permission', 'cron.edit');

        $item = Cron::query()->findOrNew($id);
        if (!$item->exists) {
            $item->is_enabled = true;
            $item->can_overlap = true;
        }

        $form = cms_create_form(CronForm::class, $item);

        $last = $item->lastRun();

        $this->layout
            ->setMenuSection('cron')
            ->setTitle(trans($id ? 'admin::cron.edit' : 'admin::cron.add'))
            ->addBreadcrumb(trans('admin::cron.list'), route('admin::cron_list'));
        return view('admin::cron.edit', compact('form', 'item', 'last'));
    }


    public function postSave(Request $request, RepositoryInterface $repo)
    {
        $this->authorize('permission', 'cron.edit');

        $this->getValidationFactory()->extendImplicit('cron_expression', function ($attr, $value) {
            return \Cron\CronExpression::isValidExpression($value);
        });

        $itemId = $request->input('id');
        $item = Cron::query()->firstOrNew(['id' => $itemId]);
        $form = cms_create_form(CronForm::class, $item);
        $validData = $form->validate($request, [], [
            'cron_expression' => trans('admin::cron.expression_invalid'),
        ]);

        $item->fill($validData);
        if (!$item->exists) {
            //
        }
        event('admin.model.cron.saving', $item);
        $item->save();
        event('admin.model.cron.saved', $item);

        $repo->cleanCache();

        $this->layout->addNotify('success', trans('admin::cron.saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::cron_edit', $item->id);
        }

        return redirect()->route('admin::cron_list');
    }

    public function action(Request $request, RepositoryInterface $repo)
    {
        $action = $request->input('action');
        $ids = $request->input('ids');
        if (!$ids) {
            return;
        }

        if ($action == 'enable') {
            $this->authorize('permission', 'cron.edit');

            foreach ($ids as $id) {
                $item = Cron::query()->find($id);
                if (!$item) {
                    continue;
                }
                $item->is_enabled = 1;
                event('admin.model.cron.saving', $item);
                $item->save();
                event('admin.model.cron.saved', $item);
            }
        } elseif ($action == 'disable') {
            $this->authorize('permission', 'cron.edit');

            foreach ($ids as $id) {
                $item = Cron::query()->find($id);
                if (!$item) {
                    continue;
                }
                event('admin.model.cron.saving', $item);
                $item->is_enabled = 0;
                event('admin.model.cron.saved', $item);
                $item->save();
            }
        } elseif ($action == 'delete') {
            $this->authorize('permission', 'cron.delete');

            foreach ($ids as $id) {
                $item = Cron::query()->find($id);
                if (!$item) {
                    continue;
                }
                event('admin.model.cron.deleting', $item);
                $item->delete();
                event('admin.model.cron.deleted', $item);
            }
        }
        $repo->cleanCache();
    }
}
