<?php

namespace AltSolution\Admin\Modules;

use AltSolution\Admin\Http\Controllers;
use AltSolution\Admin\System\AbstractModule;
use AltSolution\Admin\System\EntityBuilder;
use AltSolution\Admin\System\MenuBuilder;
use AltSolution\Admin\System\OptionBuilder;
use AltSolution\Admin\System\PermissionBuilder;
use AltSolution\Admin\System\SeoBuilder;
use Illuminate\Routing\Router;

class CoreModule extends AbstractModule
{
    public function buildMenu(MenuBuilder $builder)
    {
        $builder->addSection(trans('admin::dashboard.ms_name'), 'fas fa-tachometer-alt', route('admin::dashboard'))
            ->setAlias('dashboard')
            ->setWeight(500);

        $builder->addSection(trans('admin::debug.ms_name'), 'fas fa-bug')
            ->setAlias('debug')
            ->setWeight(-101);
        $builder->addItem(trans('admin::debug.mi_email_test'), route('admin::email_test'))
            ->setPermission('debug');
        $builder->addItem(trans('admin::debug.mi_phpinfo'), route('admin::phpinfo'))
            ->setPermission('debug');
        $builder->addItem(trans('admin::debug.mi_php'), route('admin::php'))
            ->setPermission('debug');
        $builder->addItem(trans('admin::debug.mi_sql'), route('admin::sql'))
            ->setPermission('debug');
        $builder->addItem(trans('admin::debug.mi_artisan'), route('admin::artisan'))
            ->setPermission('debug');
        $builder->addItem(trans('admin::backup.mi_name'), route('admin::backup'))
            ->setPermission('backup');
    }

    public function buildPermissions(PermissionBuilder $builder)
    {
        $builder->addSection('core', 'admin::core.ps_core')
            ->setWeight(-100);
        $builder->addPermission('admin', 'admin::core.p_admin');
        $builder->addPermission('debug', 'admin::core.p_debug');
        $builder->addPermission('cache', 'admin::core.p_cache');
        $builder->addPermission('backup', 'admin::backup.p_name');
    }

    public function buildEntity(EntityBuilder $builder)
    {
        $builder->addUrl('admin::core.me_main', '/')
            ->setWeight(100);
        $builder->addCustom('admin::core.me_custom')
            ->setWeight(-100);
    }

    public function buildSeo(SeoBuilder $builder)
    {
        if (config('seo.enable_common')) {
            $builder->addSection('cms_common', 'admin::seo.ss_common')
                ->setWeight(100);
            $builder->addField('site_name', [
                'type' => 'text',
                'label' => 'admin::seo.sf_common_name',
            ]);
            $builder->addDefaultFields();
        }
    }

    public function buildOptions(OptionBuilder $builder)
    {
        $builder->addSection('admin', 'admin::option.os_admin')
            ->setWeight(100);
        $builder->addField('email', [
            'type' => 'email',
            'label' => trans('admin::option.of_admin_email'),
            'help' => trans('admin::option.oh_admin_email'),
            'required' => true,
        ]);
        $builder->addField('email_d', [
            'type' => 'text',
            'label' => trans('admin::option.of_admin_email_d'),
            'help' => trans('admin::option.oh_admin_email_d'),
        ]);
        $builder->addField('start_page', [
            'type' => 'select',
            'label' => trans('admin::option.of_admin_start_page'),
            'help' => trans('admin::option.oh_admin_start_page'),
            'choices' => function () {
                // TODO: refactoring extract method
                $menu = cms_system()->getLayout()->getMenu();
                $result = [];
                foreach ($menu->getSections() as $section) {
                    $options = [];
                    if ($section->hasItems()) {
                        foreach ($section->getItems() as $item) {
                            $options[$item->getRelativeLink()] = $item->getName();
                        }
                    } else {
                        $options[$section->getRelativeLink()] = $section->getName();
                    }
                    $result[$section->getName()] = $options;
                }
                return $result;
            },
        ]);
    }

    public function registerRoutes(Router $router)
    {
        $router->get('/', [Controllers\IndexController::class, 'index'])->name('index');
        $router->get('dashboard', [Controllers\IndexController::class, 'dashboard'])->name('dashboard');
        $router->post('dashboard/action', [Controllers\IndexController::class, 'action'])->name('dashboard_action');
        $router->get('_upload/{file}/dl', [Controllers\UploadController::class, 'download'])->name('upload_dl');

        $router->get('elfinder', [Controllers\ElFinderController::class, 'index'])->name('elfinder');
        $router->get('elfinder/connector', [Controllers\ElFinderController::class, 'connect'])->name('elfinder_connector');
        $router->post('elfinder/connector', [Controllers\ElFinderController::class, 'connect'])->name('elfinder_connector_fulfil');

        $router->get('cache', [Controllers\CoreController::class, 'getCache'])->name('cache');
        $router->post('cache', [Controllers\CoreController::class, 'postCache'])->name('cache_fulfil');

        $router->get('debug/email_test', [Controllers\DebugController::class, 'emailTest'])->name('email_test');
        $router->post('debug/email_test', [Controllers\DebugController::class, 'emailTestPost'])->name('email_test_post');
        $router->get('debug/php', [Controllers\DebugController::class, 'php'])->name('php');
        $router->post('debug/php', [Controllers\DebugController::class, 'php'])->name('php_fulfil');
        $router->get('debug/phpinfo', [Controllers\DebugController::class, 'phpinfo'])->name('phpinfo');
        $router->get('debug/sql', [Controllers\DebugController::class, 'sql'])->name('sql');
        $router->post('debug/sql', [Controllers\DebugController::class, 'sql'])->name('sql_fulfil');
        $router->get('debug/artisan', [Controllers\DebugController::class, 'artisan'])->name('artisan');
        $router->post('debug/artisan/rpc', [Controllers\DebugController::class, 'artisanRpc'])->name('artisan_rpc');

        $router->get('backup', [Controllers\BackupController::class, 'index'])->name('backup');
        $router->post('backup', [Controllers\BackupController::class, 'postIndex'])->name('backup_fulfil');
    }
}